/*
 *
 *
 */
#ifndef __SIM_H__
#define __SIM_H__

#define SIM2181_VER "0.10"

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <readline/readline.h>
#include <readline/history.h>
#include <errno.h>

struct _ushort14 {
  unsigned short n:14;
};
typedef struct _ushort14 ushort14;
struct _short14 {
  short n:14;
};
typedef struct _short14 short14;


#include "alu.h"
#include "bdma.h"
#include "boot.h"
#include "commands.h"
#include "countstack.h"
#include "decode.h"
#include "dm.h"
#include "init.h"
#include "irq.h"
#include "loopstack.h"
#include "ops.h"
#include "pcstack.h"
#include "pm.h"
#include "pretty.h"
#include "profiler.h"
#include "reg.h"
#include "sequencer.h"
#include "statestack.h"
#include "wordops.h"

#define PM_SIZE (0x6000*3) /* base + PMOVLAY=0 + PMOVLAY=1 */
#define DM_SIZE (0x6000*2) /* base + DMOVLAY=1 + DMOVLAY=2 */
#define BDMASRC_SIZE 0x4000

#define MMAPREG_PFTYPE 0x3fe6
#define MMAPREG_PFDATA 0x3fe5
#define MMAPREG_BWCOUNT 0x3fe4
#define MMAPREG_BDMACTL 0x3fe3
#define MMAPREG_BEAD 0x3fe2
#define MMAPREG_BIAD 0x3fe1
#define MMAPREG_IDMACTL 0x3fe0

#define MMAPREG_SYSCTL 0x3fff

/* 1=true, 0=false */
#define ASTAT_AZ 0x01 /* ALU Result Zero -- Updated by all ALU but DIVS/Q */
#define ASTAT_AN 0x02 /* ALU Result Negative -- All but DIVS/Q */
#define ASTAT_AV 0x04 /* ALU Overflow -- All but DIVS/Q */
#define ASTAT_AC 0x08 /* ALU Carry -- All but DIVS/Q */
#define ASTAT_AS 0x10 /* ALU X Input Sign -- Only ABS */
#define ASTAT_AQ 0x20 /* ALU Quotient -- Only on divides */
#define ASTAT_MV 0x40 /* MAC Overflow -- Any MAC except SAT MR */
#define ASTAT_SS 0x80 /* Shifter Input Sign -- Shifter EXP op */

#define MSTAT_SECREG 0x01
#define MSTAT_BITREV 0x02
#define MSTAT_AVLATCH 0x04
#define MSTAT_ARSAT 0x08
#define MSTAT_MMODE 0x10
#define MSTAT_TIMER 0x20
#define MSTAT_GMODE 0x40

#define SSTAT_PC_EMPTY 0x01
#define SSTAT_PC_OVERFLOW 0x02
#define SSTAT_COUNT_EMPTY 0x04
#define SSTAT_COUNT_OVERFLOW 0x08
#define SSTAT_STATUS_EMPTY 0x10
#define SSTAT_STATUS_OVERFLOW 0x20
#define SSTAT_LOOP_EMPTY 0x40
#define SSTAT_LOOP_OVERFLOW 0x80

/*
 * PORTABILITY: we assume unsigned short to be 16bits.
 */

struct alu_s {
  unsigned short ax0;
  unsigned short ax1;
  unsigned short ay0;
  unsigned short ay1;
  unsigned short ar; /*16 most likely */
  unsigned short af; /* guessing 16 */
};

struct mac_s {
  unsigned short mx0;
  unsigned short mx1;
  unsigned short my0;
  unsigned short my1;
  unsigned short mr0; /* 16 */
  unsigned short mr1; /* 16 */
  unsigned char mr2; /* 8 */
  unsigned long long mf; /* huh? */
};

struct shifter_s {
  unsigned short si;
  unsigned char se;
  unsigned char sb:5;
  unsigned short sr1; /* are srx linked? */
  unsigned short sr0;
};

struct loop_stack_entry_s {
  unsigned short addr:14;
  unsigned short cond:4;
};

struct count_stack_entry_s {
  unsigned short addr:14;
};

struct pc_stack_entry_s {
  unsigned short addr:14;
};

struct status_stack_entry_s {
  unsigned char astat:8; /* 8bit */
  unsigned short mstat:7; /* size? 7bit */
  unsigned short imask:10; /* size? 10bit */
};

struct sequencer_s {
  unsigned int curinstr:24; /* not part of real sequencer */
  unsigned short pc:14; /* 14bit */
  unsigned char icntl:5;
  unsigned char sstat:8; /* 8bit */
  unsigned short mstat:7; /* size? 7bit */
  unsigned short imask:10; /* size? 10bit */
  unsigned char astat:8; /* 8bit */
  unsigned short cntr:14;
  unsigned short ifc; /* size? 16bit */
  struct loop_stack_entry_s loop_stack[4]; /* pg3-4; 18 x 4bit*/
  unsigned short loop_stack_len;
  struct count_stack_entry_s count_stack[4]; /*14bit x 4w; pg3-5 */
  unsigned short count_stack_len;
  struct pc_stack_entry_s pc_stack[16]; /* 14bit x 16 */
  unsigned short pc_stack_len;
  struct status_stack_entry_s status_stack[16]; /* 8+7+10bit x 16 */
  unsigned short status_stack_len;
};

struct dsp_s {
  unsigned char pm[PM_SIZE]; /* NEVER access directly--paging done elsewhere */
  unsigned char dm[DM_SIZE]; /* NEVER access directly */
  unsigned char bdmasrc[BDMASRC_SIZE]; 
  struct {
    struct alu_s pri;
    struct alu_s sec;
    struct alu_s *cur;
  } alu;
  struct {
    struct mac_s pri;
    struct mac_s sec;
    struct mac_s *cur;
  } mac;
  struct {
    struct shifter_s pri;
    struct shifter_s sec;
    struct shifter_s *cur;
  } shifter;
  struct sequencer_s sequencer;
  struct {
    struct {
      short14 i0;
      short14 i1;
      short14 i2;
      short14 i3;
      short14 l0;
      short14 l1;
      short14 l2;
      short14 l3;
      short14 m0;
      short14 m1;
      short14 m2;
      short14 m3;
      short14 _b0; /* these are used internally only -- base address */
      short14 _b1;
      short14 _b2;
      short14 _b3;
    } dag1;
    struct { 
      short14 i4;
      short14 i5;
      short14 i6;
      short14 i7;
      short14 l4;
      short14 l5;
      short14 l6;
      short14 l7;
      short14 m4;
      short14 m5;
      short14 m6;
      short14 m7;
      short14 _b4;
      short14 _b5;
      short14 _b6;
      short14 _b7;
    } dag2;
  } dag;
  struct {
    unsigned char px;
  } bus_exchanger;
  struct {
    unsigned char dmovlay:3;
    unsigned char pmovlay:3;
  } memory_interface;  /* memory mapped values are not stored here */
  struct {
    unsigned int rx0; /* size? */
    unsigned int tx0; /* size? */
  } sport0;
  struct irq_s irqs[IRQCOUNT];
};

extern struct dsp_s curdsp;

#endif /* __SIM_H__ */
